﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using CommunityToolkit.Mvvm.ComponentModel;
using CtrEditor.Siemens;
using CtrEditor.Simulacion;
using SkiaSharp;

namespace CtrEditor.ObjetosSim
{
    /// <summary>
    /// Interaction logic for ucTransporteGuiasUnion.xaml
    /// </summary>
    public partial class osTransporteGuiasUnion : osBase, IosBase
    {
        private osBase _osMotorA = null;
        private osBase _osMotorB = null;

        Dictionary<Rectangle, simTransporte> SimGeometriaT;
        Dictionary<Rectangle, simGuia> SimGeometriaG;
        Dictionary<Rectangle, Storyboard> Storyboards;
        Dictionary<Rectangle, BoolReference> TransportsDirection;
        Dictionary<Rectangle, FloatReference> TransportsVelocidad;

        public static string NombreClase()
        {
            return "Transporte Guias Union";
        }
        private string nombre = NombreClase();
        public override string Nombre
        {
            get => nombre;
            set => SetProperty(ref nombre, value);
        }

        [ObservableProperty]
        public string motorA;

        partial void OnMotorAChanged(string value)
        {
            _osMotorA = ObtenerLink(MotorA, typeof(osVMmotorSim));
        }

        [ObservableProperty]
        public float velocidadActualA;

        partial void OnVelocidadActualAChanged(float value)
        {
            if (_visualRepresentation is ucTransporteGuiasUnion uc)
            {
                var transporte = uc.TransporteA;
                SetSpeed(transporte);
            }
        }

        [ObservableProperty]
        bool invertirDireccionA;

        partial void OnInvertirDireccionAChanged(bool value)
        {
            if (_visualRepresentation is ucTransporteGuiasUnion uc)
            {
                var transporte = uc.TransporteA;

                SetSpeed(transporte);
                ActualizarStoryboards(transporte);
            }
        }



        [ObservableProperty]
        public string motorB;

        partial void OnMotorBChanged(string value)
        {
            _osMotorB = ObtenerLink(MotorB, typeof(osVMmotorSim));
        }

        [ObservableProperty]
        public float velocidadActualB;

        partial void OnVelocidadActualBChanged(float value)
        {
            if (_visualRepresentation is ucTransporteGuiasUnion uc)
            {
                var transporte = uc.TransporteB;
                SetSpeed(transporte);
            }
        }

        [ObservableProperty]
        bool invertirDireccionB;

        partial void OnInvertirDireccionBChanged(bool value)
        {
            if (_visualRepresentation is ucTransporteGuiasUnion uc)
            {
                var transporte = uc.TransporteB;

                SetSpeed(transporte);
                ActualizarStoryboards(transporte);
            }
        }


        [ObservableProperty]
        public float anchoRecto;
        [ObservableProperty]
        public float anchoCentral;

        [ObservableProperty]
        public float alto;

        partial void OnAltoChanged(float value)
        {
            ActualizarGeometrias();
        }

        [ObservableProperty]
        public float angulo;
        [ObservableProperty]
        public float frictionCoefficient;
        [ObservableProperty]
        public float velMax50hz;
        [ObservableProperty]
        public float tiempoRampa;
        [ObservableProperty]
        public bool esMarcha;
        [ObservableProperty]
        private float distance;
        [ObservableProperty]
        private float altoGuia;

        void ActualizarStoryboards(Rectangle transporte)
        {
            if (!Storyboards.Keys.Contains(transporte)) return;

            var direccion = TransportsDirection[transporte].Value;
            var velocidad = TransportsVelocidad[transporte].Value;

            Storyboards[transporte] = CrearAnimacionMultiStoryBoardTrasnporte(Storyboards[transporte], transporte, direccion);
            ActualizarAnimacionMultiStoryBoardTransporte(Storyboards[transporte], velocidad);
        }

        void SetSpeed(Rectangle transporte)
        {
            if (!Storyboards.Keys.Contains(transporte)) return;

            var invertirDireccion = TransportsDirection[transporte].Value;
            var velocidad = TransportsVelocidad[transporte].Value;

            if (invertirDireccion)
                SimGeometriaT[transporte]?.SetSpeed(-velocidad);
            else
                SimGeometriaT[transporte]?.SetSpeed(velocidad);

            ActualizarAnimacionMultiStoryBoardTransporte(Storyboards[transporte], velocidad);
        }

        private void ActualizarGeometrias()
        {
            if (_visualRepresentation is ucTransporteGuiasUnion uc)
            {
                foreach (var transporte in SimGeometriaT)
                {
                    UpdateRectangle(transporte.Value, transporte.Key, Alto, Ancho, Angulo);
                    ActualizarStoryboards(transporte.Key);
                }
                foreach (var l in SimGeometriaG)
                    UpdateOrCreateLine(l.Value, l.Key);

            }
        }


        public osTransporteGuiasUnion()
        {
            Ancho = 1;
            Alto = 0.10f;
            AltoGuia = 0.03f;
            Distance = 0.01f;
            SimGeometriaT = new Dictionary<Rectangle, simTransporte>();
            SimGeometriaG = new Dictionary<Rectangle, simGuia>();
            Storyboards = new Dictionary<Rectangle, Storyboard>();
            TransportsDirection = new Dictionary<Rectangle, BoolReference>();
            TransportsVelocidad = new Dictionary<Rectangle, FloatReference>();
        }

        public override void UpdateGeometryStart()
        {
            // Se llama antes de la simulacion
            ActualizarGeometrias();
        }

        public override void SimulationStop()
        {
            // Se llama al detener la simulacion
            if (_visualRepresentation is ucTransporteGuiasUnion uc)
            {
                SetSpeed(uc.TransporteB);
                SetSpeed(uc.TransporteA);
            }
        }

        public override void UpdatePLC(PLCModel plc, int elapsedMilliseconds)
        {
            if (_osMotorA != null)
            {
                if (_osMotorA is osVMmotorSim motor)
                    VelocidadActualA = motor.Velocidad;
            }
            else if (MotorA.Length > 0)
                _osMotorA = ObtenerLink(MotorA, typeof(osVMmotorSim));

            if (_osMotorB != null)
            {
                if (_osMotorB is osVMmotorSim motor)
                    VelocidadActualB = motor.Velocidad;
            }
            else if (MotorB.Length > 0)
                _osMotorB = ObtenerLink(MotorB, typeof(osVMmotorSim));

        }

        public override void ucLoaded()
        {
            // El UserControl ya se ha cargado y podemos obtener las coordenadas para 
            // crear el objeto de simulacion
            ActualizarLeftTop();

            // El UserControl ya se ha cargado y podemos obtener las coordenadas para 
            // crear el objeto de simulacion
            if (_visualRepresentation is ucTransporteGuiasUnion uc)
            {
                var connector = new RectangleConnector(uc.Canvas, MeterToPixels(Alto/2));
                connector.ConnectRectangles();

                foreach (var child in uc.Canvas.Children)
                {
                    if (child is Rectangle rect)
                    {
                        if (rect.Name.StartsWith("Transporte"))
                        {
                            SimGeometriaT.Add(rect,AddRectangle(simulationManager, rect, Alto, Ancho, Angulo));
                            Storyboards.Add(rect,CrearAnimacionMultiStoryBoardTrasnporte(rect,false));                            
                        }
                        if (rect.Name.StartsWith("Guia"))
                            SimGeometriaG.Add(rect,AddLine(simulationManager, rect));
                    }
                }

                TransportsDirection.Add(uc.TransporteA, new BoolReference(() => InvertirDireccionA, value => InvertirDireccionA = value));
                TransportsDirection.Add(uc.TransporteB, new BoolReference(() => InvertirDireccionB, value => InvertirDireccionB = value));

                TransportsVelocidad.Add(uc.TransporteA, new FloatReference(() => VelocidadActualA, value => VelocidadActualA = value));
                TransportsVelocidad.Add(uc.TransporteB, new FloatReference(() => VelocidadActualB, value => VelocidadActualB = value));

            }
            OnMotorAChanged(MotorA);
        }


        public override void ucUnLoaded()
        {
            // El UserControl se esta eliminando
            // eliminar el objeto de simulacion
            foreach (var s in SimGeometriaT)
                simulationManager.Remove(s.Value);
            foreach (var s in SimGeometriaG)
                simulationManager.Remove(s.Value);
        }


    }

    public class BoolReference
    {
        private Func<bool> getter;
        private Action<bool> setter;

        public BoolReference(Func<bool> getter, Action<bool> setter)
        {
            this.getter = getter;
            this.setter = setter;
        }

        public bool Value
        {
            get => getter();
            set => setter(value);
        }
    }

    public class FloatReference
    {
        private Func<float> getter;
        private Action<float> setter;

        public FloatReference(Func<float> getter, Action<float> setter)
        {
            this.getter = getter;
            this.setter = setter;
        }

        public float Value
        {
            get => getter();
            set => setter(value);
        }
    }

    public class RectangleConnector
    {
        private Canvas canvas;
        private double maxDistance;

        public RectangleConnector(Canvas canvas, double maxDistance)
        {
            this.canvas = canvas;
            this.maxDistance = maxDistance;
        }

        public void ConnectRectangles()
        {
            var rectangles = canvas.Children.OfType<Rectangle>().Where(rect => rect.Name.StartsWith("Guia")).ToList();

            foreach (var rect in rectangles)
            {
                var connections = GetConnectionPoints(rect);
                foreach (var point in connections)
                {
                    var closestPoint = GetClosestPoint(point, rectangles, rect);
                    if (closestPoint != null && GetDistance(point, closestPoint.Value) < maxDistance)
                    {
                        AdjustRectanglePosition(rect, point, closestPoint.Value);
                    }
                }
            }
        }

        private List<Point> GetConnectionPoints(Rectangle rect)
        {
            double width = rect.Width;
            double height = rect.Height;
            double angle = GetRotationAngle(rect);

            // Puntos de conexión sin rotación
            var points = new List<Point>
        {
            new Point(0, height / 2),
            new Point(width, height / 2)
        };

            // Aplicar rotación
            var rotatedPoints = points.Select(p => RotatePoint(p, angle, new Point(width / 2, height / 2))).ToList();
            return rotatedPoints;
        }

        private Point RotatePoint(Point point, double angle, Point center)
        {
            double radians = angle * Math.PI / 180;
            double cos = Math.Cos(radians);
            double sin = Math.Sin(radians);

            double dx = point.X - center.X;
            double dy = point.Y - center.Y;

            double x = center.X + (dx * cos - dy * sin);
            double y = center.Y + (dx * sin + dy * cos);

            return new Point(x, y);
        }

        private double GetRotationAngle(Rectangle rect)
        {
            var transform = rect.RenderTransform as RotateTransform;
            return transform?.Angle ?? 0;
        }

        private Point? GetClosestPoint(Point point, List<Rectangle> rectangles, Rectangle excludeRect)
        {
            Point? closestPoint = null;
            double closestDistance = double.MaxValue;

            foreach (var rect in rectangles)
            {
                if (rect == excludeRect) continue;

                var points = GetConnectionPoints(rect);
                foreach (var p in points)
                {
                    double distance = GetDistance(point, p);
                    if (distance < closestDistance)
                    {
                        closestDistance = distance;
                        closestPoint = p;
                    }
                }
            }

            return closestPoint;
        }

        private double GetDistance(Point p1, Point p2)
        {
            return Math.Sqrt(Math.Pow(p2.X - p1.X, 2) + Math.Pow(p2.Y - p1.Y, 2));
        }

        private void AdjustRectanglePosition(Rectangle rect, Point currentPoint, Point targetPoint)
        {
            double offsetX = targetPoint.X - currentPoint.X;
            double offsetY = targetPoint.Y - currentPoint.Y;

            double newLeft = Canvas.GetLeft(rect) + offsetX;
            double newTop = Canvas.GetTop(rect) + offsetY;

            Canvas.SetLeft(rect, newLeft);
            Canvas.SetTop(rect, newTop);
        }
    }

    public partial class ucTransporteGuiasUnion : UserControl, IDataContainer
    {
        public osBase? Datos { get; set; }

        public ucTransporteGuiasUnion()
        {
            InitializeComponent();
            this.Loaded += OnLoaded;
            this.Unloaded += OnUnloaded;
        }
        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucLoaded();
        }
        private void OnUnloaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucUnLoaded();
        }
        public void Resize(float width, float height)
        {
            if (Datos is osTransporteGuiasUnion datos)
                datos.Ancho = PixelToMeter.Instance.calc.PixelsToMeters(width);
        }
        public void Move(float LeftPixels, float TopPixels)
        {
            if (Datos != null)
            {
                Datos.Left = PixelToMeter.Instance.calc.PixelsToMeters(LeftPixels);
                Datos.Top = PixelToMeter.Instance.calc.PixelsToMeters(TopPixels);
            }
        }
        public void Rotate(float Angle)
        {
            if (Datos != null)
                if (Datos is osTransporteGuiasUnion datos)
                    datos.Angulo = Angle;
        }
        public void Highlight(bool State) { }
        public int ZIndex()
        {
            return 1;
        }

    }
}
